#include <iom168v.h>
#include <stdlib.h>
#include <string.h>
#include "LED_Display.h"

#define LED_B_CONTROL_MASK ~((1<<4)|(1<<5))
#define LED_D_CONTROL_MASK ~((1<<2) | (1<<3) | (1<<4) | (1<<5) | (1<<6) | (1<<7))

#define LED_B_DATA_MASK ((1<<0) | (1<<1) | (1<<2) | (1<<3))
#define LED_C_DATA_MASK ((1<<0) | (1<<1) | (1<<2) | (1<<3))
	 
//The arrays below contain the port control lines for each of the 7 segment displays 
const char LED_B_CONTROL[] = {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20, 0x10};
const char LED_D_CONTROL[] = {0x04, 0x08, 0x10, 0x20, 0x40, 0x80, 0x00, 0x00};
//0-F
const unsigned char numbers[] = {0x50,0xF9,0xC4,0xE0,0x69,0x62,0x42,0xF8,0x40,0x68,0x48,0x43,0x56,0xC1,0x46,0x4E};

//
unsigned char display[LINES][WIDTH];

/*
 *Call this function to display an float on the 7segment display
 *decimalAmount will set how many decimal places will be displayed
 */
void Display_Float(float fvalue,short line, short decimalAmount)
{
	char *outstrg;
	unsigned char text[WIDTH];
	int i,j=0;

	
	//If we try to write to an invalid line number
	if (line >= LINES)
	{
	  	return;
	}
	 
	if(fvalue>0) fvalue += 0.00001;
	if(fvalue<0) fvalue -= 0.00001;
	
	outstrg = ftoa(fvalue,&i);
	
	for(i=0; i < WIDTH; i++)
	{
	 	if( j< strlen(outstrg))
		{
	 	 		if(outstrg[j]=='.')
				{
				 	 i--;//we are working on the last digit
		    	 	 text[i] = text[i]&DECIMAL_POINT;
					 //how far we want to shift it
					 decimalAmount = WIDTH - (decimalAmount+i+1);
				}
				else if(outstrg[j]=='-')
				{
		    	 	 text[i] = MINUS;
				}
				else
				{
		 		 	text[i] = numbers[outstrg[j]-'0'];
					
			
				}
				j++;
		}
		else
		{
		 	text[i] = numbers[0];//fill in zeros
		}
	}
	
	
	for(i = WIDTH-1; i >= 0 ; i--)
	{
			if(i-decimalAmount>=0)
			{
					display[line][i] = text[i-decimalAmount];
			}
			else
			{
					display[line][i] = BLANK_SPACE;
			}
	}
}

/*
 *Call this function to display an integer on the 7segment display
 */
void Display_Int(int value, short line, short base)
{
 	 unsigned char text[WIDTH];
 
 	 short start=0, count = 0,end;
	 unsigned char temp;
	 
	 //if we try to write to a line that does not exist, do nothing
	 if (line >= LINES)
	 {
	  	return;
	 }
	 
	 if (base == DEC)
	 {
		 if(value < 0)
		 {	 
			text[count++] = MINUS;
			value = -value;
			start++;
		 } 
	 }
	 else if (base == HEX)
	 {
			text[count++] = DISPLAY_H;
			start++;
	 }
	 

	 
	 //find the digits
	 do {
	 	if(count <= WIDTH)
		{
		 		 text[count++] = numbers[value % base];
		}
		else//Ensure our number is not to big to display
		{
		 break;
		}
	 } while ((value /= base) > 0);
	 
	 end = --count;

	 
	 //reverse to correct order
	 while (start <= count)
	 {
		 temp = text[start];
		 text[start] = text[count];
		 text[count] = temp;
		 start++;
		 count--; 
	 }
	  
     if(end < WIDTH)
	 {
	  		for(count = WIDTH-1; count >= 0 ;count--)
			{
			 		if(end>=0)
					{
							display[line][count] = text[end--];
					}
					else
					{
							display[line][count] = BLANK_SPACE;
					}
			}
	 }
	 else//If our number was to large to display display --------
	 {
	  	 for(count=0; count < WIDTH;count++)
		 {
		  		  display[line][count] = MINUS;
		 }
	 }
}

/*
 *Call this function to set your port directions
 */
void LED_Init(void)
{
 	 DDRB |= (~LED_B_CONTROL_MASK) | LED_B_DATA_MASK;
 	 DDRC |= LED_C_DATA_MASK;
 	 DDRD |= ~LED_D_CONTROL_MASK;

}

void LED_Clear(void)
{
 	 short i, j;
	 
 	 for(i = 0; i < LINES; i++)
 	 {
  	   for(j = 0; j < WIDTH; j++)
	   {
	   		 display[i][j] = BLANK_SPACE;
	   }
 	 }
}

/*
 *Call this function to light one segment of the display
 *The next time the function is called, the next segment
 *will be lit
 */
void LED_Update(void)
{
 	 static unsigned char digit = 0;
	 
	 //Turn off the segments
	 PORTB |= LED_B_DATA_MASK;
	 PORTC |= LED_C_DATA_MASK;
	 
	 //Turn off the last Digit
	 PORTB &= LED_B_CONTROL_MASK;
	 PORTD &= LED_D_CONTROL_MASK;
	 
	 //Turn on the the new Digit
	 PORTB |= LED_B_CONTROL[digit];
	 PORTD |= LED_D_CONTROL[digit];

	 //Turn on the current data
	 PORTC &= (display[0][digit]&0xF) | 0xF0;
	 //PORTB &= (numbers[digit]&0xF0)>>4;
	 PORTB &= ((display[0][digit]&0xF0)>>4) | 0xF0;
 
	 //Update the next digit next time
	 digit++;
	 
	 if(digit >= WIDTH)
	 {
	  	digit = 0;
	 }
}