#ifndef RINGBUFFER_H_
#define RINGBUFFER_H_

/* Includes: */
    
#include <iom168v.h>
#include <macros.h>

/* Buffer Configuration */ 

/* Size of ringbuffer */
#define BUFF_LENGTH 32      // Set to the static ringbuffer size for all ringbuffers (place size after define)

/* Drop mode - select behaviour when Buffer_StoreElement called on a full buffer: */
#define BUFF_DROPOLD             // Uncomment to cause full ring buffers to drop the oldest character to make space when full
// #define BUFF_DROPNEW          // Uncomment to cause full ring buffers to drop the new character when full
// #define BUFF_NODROPCHECK      // Uncomment to ignore full ring buffer checks - checking left to user!

/* Underflow behaviour - select behaviour when Buffer_GetElement is called with an empty ringbuffer: */
//#define BUFF_EMPTYRETURNSZERO  // Uncomment to return 0 when an empty ringbuffer is read
#define BUFF_NOEMPTYCHECK        // Uncomment to disable checking of empty ringbuffers - checking left to user!
	
/* Buffer storage type - set the datatype for the stored data */
#define BUFF_DATATYPE unsigned char 

/* Buffer storage counter */
#define RingBuff_Elements_t char
	 	


	


/* Type Defines: */
typedef BUFF_DATATYPE RingBuff_Data_t;

typedef volatile struct
{
	RingBuff_Data_t      Buffer[BUFF_LENGTH];
	RingBuff_Data_t*     InPtr;
	RingBuff_Data_t*     OutPtr;
	RingBuff_Elements_t  Elements;
} RingBuff_t;
		
/* Function Prototypes: */
void Buffer_Initialize(RingBuff_t* Buff);
void Buffer_StoreElement(RingBuff_t* Buffer, RingBuff_Data_t Data);
RingBuff_Data_t Buffer_GetElement(RingBuff_t* Buffer);
RingBuff_Data_t Buffer_PeekElement(const RingBuff_t* Buffer);

	
#endif
