/**************************************************************

   Program Title:  Led8 Controller
   
   Program Name:  LED8_SPI_Flash.c	Version: 0.85
   
   Author:  SS	  					Date:  August 10, 2005
   
   Compiled with ImageCraft ICCAVR Standard V7.02
   
   The AVR SPI lines are connected to two serial flash devices U2 
   and U5. If the AVR needs to communicate with the Serial 
   Flash devices the Blackfin must relinquish control of the SPI 
   lines. 
   
   Serial flash devices compatible with the P1 are the AT45DB041
   (512kx8), AT45DB081 (1Mx8), AT45DB161 (2Mx8), AT45DB321 (4Mx8)
   or AT45DB642 (8Mx8).
   
   The Serial Flash chips are page oriented with 2 SRAM buffers of 264,
   528 or 1024 bytes each.  The AVR programs the serial flash devices by 
   loading files via the serial line (in IntelHex32 format) into the RAM 
   buffers before transfer to the appropriate page.  Page erase/programming 
   requires about 30msec per page. The routines in this file perform these 
   functions.  The host terminal program must support XON/XOFF flow control.
   
   Copyright 2004-2006, SOC Robotics, Inc.  All rights reserved.
   
****************************************************************/
/*
 * Copyright 2004-2006, SOC Robotics, Inc.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 * 4. Redistribution is only allowed in combination with SOC Machines, 
 *    Inc. hardware products.
 *
 * THIS SOFTWARE IS PROVIDED BY SOC MACHINES, INC. AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL OC MACHINES, 
 * INC. OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * For additional information see http://www.soc-robotics.com
 */

/*
 * $Log: Wasp_Monitor.c,v $
 * Revision 0.86  2006/02/07 16:40:20  SS
 *    2-wire interface, 8MHz clock, Serial Flash Writer, XON/XOFF
 * Revision 0.52  2005/06/04 18:36:42  SS
 * Preview release
 *
 */
#include <iom48v.h>
#include <macros.h>
#include <string.h>
#include <stdlib.h>
//#include <stdio.h>
#include "LED8_Uart.h"
#include "LED8_Defines.h"
#include "LED8_Extern.h"

// Set Serial Flash chip select state
void FlashChipSelect(unsigned char mode)
{
     shortdelay(2);
	 if(mode==LOW) PORTB &= 0xEF; 
	 else PORTB |= 0x10;  
     shortdelay(2);
}

//  SPI Flash device address construction function
//  Read address consists of 4 zero bits+11 page bits+9 byte start bits
void ConvertFlashAddress(unsigned int address)
{
 	int pages,bytestart,superpages;
	
	pages = address/flashpagesize0;
	superpages = pages/7;
	bytestart = address - pages*flashpagesize0;
    FlashAddress[0] = superpages;
	FlashAddress[1] = ((pages&0x00ff)<<1) + (bytestart>>8);
	FlashAddress[2] = bytestart & 0x00FF;
}

//  Get serial flash chip status
unsigned char SPI_SerialFlashStatus(void)
{
 	 unsigned char instatus,instatus2;
	 
	 //  Set PB4 - Flash CS
	 FlashChipSelect(LOW);

	 // Send read status command
 	 SPI_MasterTransmit(SF_READ_STATUS);

	 instatus = SPI_MasterReceive();
	 instatus = SPI_MasterReceive();
	 
	 // Set PC4 or PD2 high - deselect Flash chip
	 FlashChipSelect(HIGH);
	 
	 return instatus;
}

//  SPI Flash continuous read command for AT45DB041B
//  The AT45DB041B has 2048 264byte pages
void SPI_StartReadFlashPage(unsigned int address)
{
 	 int i;
	 
	 // Send Continuous read command
 	 SPI_MasterTransmit(SF_READ_CONT);
	 
	 // Convert address to 24 bit address
	 ConvertFlashAddress(address);
	 for(i=0; i<3; i++)  SPI_MasterTransmit(FlashAddress[i]);
	 
	 //  32 dummy bits
	 for(i=0; i<4; i++)  SPI_MasterTransmit(0x00);
}

//  SPI Flash Buffer Read command for AT45DB041B
void SPI_ReadFlashBuffer(unsigned int address,unsigned int buffernumber)
{
 	 int i;
	 
	 // Send buffer read command
	 if(buffernumber==1) SPI_MasterTransmit(SF_READ_BUFFER1);
	 else SPI_MasterTransmit(SF_READ_BUFFER2);
	 
	 // Convert address to 24 bit address
	 ConvertFlashAddress(address);
	 FlashAddress[0] = 0x00;
	 for(i=0; i<3; i++)  SPI_MasterTransmit(FlashAddress[i]);
	 
	 //  8 dummy bits
	 SPI_MasterTransmit(0x00);
}


//  Display P1 Blackfin Flash memory
//    Blackfin must be reset
void DisplaySerialFlash(unsigned int address,unsigned int numbytes,char readmode)
{
 	 unsigned char i,byte,bytesonlastline,asciicnt,crcvalue,tempvalue;
	 int linecnt,numlines,newaddress;

	 //  Set PC4 low - Flash CS
	 FlashChipSelect(LOW);  

	 if(readmode==READCONTINUOUS) SPI_StartReadFlashPage(address);
	 else if(readmode==READBUFFER) SPI_ReadFlashBuffer(address,1);
//	 puts("Send Continuous Read command\n\r");
	 linecnt = asciicnt = 0;
	 bytesonline = 16;	 
	 numlines = numbytes/bytesonline;
	 bytesonlastline = numbytes - numlines*bytesonline;
	 while(linecnt<numlines) {
	 	 crcvalue = 0;
	     putchar(':');
		 //  Output bytes on line
		 PrintHex(bytesonline);
		 crcvalue += bytesonline;
	     //  Output hex start address
		 newaddress = address+linecnt*16;
	     PrintWordHex(newaddress);
		 tempvalue = (newaddress&0xff00)>>8;
		 crcvalue += tempvalue;
		 tempvalue = newaddress&0x00ff;
		 crcvalue += tempvalue;
		 //  Output 00
		 puts("00");
		 //  Output data
	     for(i=0; i<bytesonline; i++) {
	         byte = SPI_MasterReceive();
			 ascii[asciicnt++] = byte;
			 crcvalue += byte;
	         PrintHex(byte);
	         }
		 linecnt++;
		 asciicnt = 0;
		 // Output CRC
	     PrintHex((~crcvalue)+0x01);
	     puts("  ");
		 for(i=0; i<bytesonline; i++) {
		     if(ascii[i]>0x1F&&ascii[i]<0x80)  putchar(ascii[i]);
			 else putchar('.');
			 }
	     putCRLF();  
		 }
	 if(bytesonlastline) {
	     crcvalue = 0;
	     putchar(':');
		 //  Output bytes on line
		 PrintHex(bytesonline);
		 crcvalue += bytesonline;
	     //  Output hex start address
		 newaddress = address+linecnt*16;
	     PrintWordHex(newaddress);
		 tempvalue = (newaddress&0xff00)>>8;
		 crcvalue += tempvalue;
		 tempvalue = newaddress&0x00ff;
		 crcvalue += tempvalue;
		 //  Output 00
		 puts("00");
		 //  Output data
	     for(i=0; i<bytesonlastline; i++) {
	         byte = SPI_MasterReceive();
			 ascii[asciicnt++] = byte;
			 crcvalue += byte;
	         PrintHex(byte);
	         }
		 // Output CRC
	     PrintHex((~crcvalue)+0x01);
	     puts("  ");
		 for(i=0; i<bytesonlastline; i++) {
		     if(ascii[i]>0x1F&&ascii[i]<0x80)  putchar(ascii[i]);
			 else putchar('.');
			 }
		 putCRLF();
		 
		 // Set PC4 high - deselect Flash chip
		 FlashChipSelect(HIGH);  
		 }
}

// Transfer buffer n to flash memory with built-in erase
//  This operation takes about 20msec max
void  ProgramBuffertoPageErase(unsigned char bufferid)
{
     unsigned char i;
	 
	 // Serial Flash CS low
	 FlashChipSelect(LOW);    
	 
	 //  Buffer write
	 if(bufferid==1) SPI_MasterTransmit(SF_BUF1_TO_FLASH_ERASE);
	 else SPI_MasterTransmit(SF_BUF2_TO_FLASH_ERASE);
	 
	 // Convert address to 24 bit address
	 ConvertFlashAddress(inteladdress);
//	 FlashAddress[0] = 0x00;
	 for(i=0; i<3; i++)  SPI_MasterTransmit(FlashAddress[i]);
	 
	 // Serial Flash CS high - start erase
	 FlashChipSelect(HIGH);   	 
}

// Maximum erase time is 20msec
void  EraseFlashPage()
{
     unsigned char i;
	 
	 // Serial Flash CS low
	 FlashChipSelect(LOW);    
	 
	 //  Buffer write
	 SPI_MasterTransmit(SF_PAGE_ERASE);
	 
	 // Convert address to 24 bit address
	 ConvertFlashAddress(inteladdress);
//	 FlashAddress[0] = 0x00;
	 for(i=0; i<3; i++)  SPI_MasterTransmit(FlashAddress[i]);
	 
	 // Serial Flash CS high - start erase
	 FlashChipSelect(HIGH);   	 
}

//  Load data into flash chip RAM buffer
void ProgramFlashBuffer(unsigned char bufferid)
{
     unsigned char i;
	 
	 // Serial Flash CS low
	 FlashChipSelect(LOW);    
	 
	 //  Buffer write
	 if(bufferid==1)  SPI_MasterTransmit(SF_WRITE_BUFFER1);
	 else  SPI_MasterTransmit(SF_WRITE_BUFFER2);
	 
	 // Convert address to 24 bit address
	 ConvertFlashAddress(inteladdress);
//	 FlashAddress[0] = 0x00;
	 for(i=0; i<3; i++)  SPI_MasterTransmit(FlashAddress[i]);
	 
	 for(i=0; i<bytesonline; i++)  SPI_MasterTransmit(intelbinary[i]);
	 
	 // Serial Flash CS high
	 FlashChipSelect(HIGH);
}

void LoadFlashPage(unsigned char bufferid)
{
     unsigned char i;
	 
	 // Serial Flash CS low
	 FlashChipSelect(LOW);    
	 
	 //  Buffer write
	 if(bufferid==1)  SPI_MasterTransmit(SF_FLASH_TO_BUF1);
	 else  SPI_MasterTransmit(SF_FLASH_TO_BUF2);
	 
	 // Convert address to 24 bit address
	 ConvertFlashAddress(inteladdress);
//	 FlashAddress[0] = 0x00;
	 for(i=0; i<3; i++)  SPI_MasterTransmit(FlashAddress[i]);
	 
	 // Serial Flash CS high
	 FlashChipSelect(HIGH);
    
}

void  WaitForFlashFree(void)
{ 	  
	  while(!(SPI_SerialFlashStatus()&0x80)) ;
}

//  Main Serial Flash programming state controller
//
//  Serial flash chips are programmed one page at a time.  Each flash chip
//  has two SRAM buffers.  Due to AVR limitations programming data is first
//  loaded into the SRAM buffers then transferred to Flash.  SRAM buffers 
//  range in size from 264 to 1024 bytes so the state machine must keep track
//  of different buffer sizes.  IntelHex records range in size from 1 to 32
//  bytes (AVR limitation) and can arrive at any start address.
//
//  As IntelHex records are received on the serial channel they are first
//  loaded into Flash SRAM (AVR doesn't have enough SRAM available) and then
//  written to a Flash page if a termination record is received or the SRAM 
//  buffer becomes full.  To prevent Flash overwrite a Flash page is first 
//  loaded into SRAM and new data overwritten in SRAM.  
//
//  To reduce variable use bytesonline, inteladdress and intelsuperaddress
//  are global variables.
void ProgramSerialFlash(void)
{
 	 long globaladdress,shortaddress;
	 int newpage,newpage2,shortbytecnt,newbytecnt,i,firstbytesonline;
	 
	 globaladdress = intelsuperaddress*65536 + inteladdress;
	 newpage = globaladdress/flashpagesize0;
	 oldinteladdress = inteladdress;
//	 puts(" ga="); PrintWordHex(inteladdress); putchar(' ');
	 
     // Always load flash page for first record
 	 if(flashprogrammingactive==NO) {
	     totalbytecnt = 0;
		 LoadFlashPage(1);
		 WaitForFlashFree();  // wait for page transfer to complete 250usec
	     flashprogrammingactive = YES;
		 flashpageloaded = YES;
		 oldpage = newpage;
		 }

 	 // See if flash page was loaded - preserves old data not in the new
	 // record from being overwritten
 	 else if(flashpageloaded==NO)  {
	     LoadFlashPage(1);
		 WaitForFlashFree();  // wait for programming to complete - 20msec
		 flashpageloaded = YES;
		 }
		 
	 // See if number of bytes in record causes a page boundary traverse
	 //  tbd - what if buffer fills exactly?
//	 newbytecnt = bytesonline;
	 newpage2 = (globaladdress+bytesonline)/flashpagesize0;
	 shortaddress = newpage*flashpagesize0;	 
//     if((globaladdress+bytesonline-1)==(shortaddress+flashpagesize0-1)) {
//	    puts(" yureka ");
//		printf(" ad=%x ",inteladdress);
//		}

	 if(newpage!=newpage2) {
	     flashpageboundarytraversed = YES;
	     shortbytecnt = flashpagesize0 - (globaladdress - shortaddress);
	     newbytecnt = bytesonline - shortbytecnt;
//		 printf(" ad=%x ",inteladdress);
		 firstbytesonline = bytesonline;
		 bytesonline = shortbytecnt;
//		 printf(" sc=%d %d ",shortbytecnt,newbytecnt);
		 }
	 
	 totalbytecnt += bytesonline;
	 ProgramFlashBuffer(1); 
//	 printf(" lb=%d",totalbytecnt); 
	 flashdatainbuffer = YES;
	 
	 //  If Buffer full or boundary traversed transfer buffer to flash
	 if((newbytecnt==0)||(flashpageboundarytraversed==YES))  {
	     ProgramBuffertoPageErase(1);
		 WaitForFlashFree();  // wait for programming to complete - 20msec
		 flashpageloaded = NO;
		 flashdatainbuffer = NO;
//		 printf(" fp=%d",newpage); 
		 }
	 
	 //  If record caused a page boundary traversal load next page
	 if(flashpageboundarytraversed==YES) {
	     //newpage++;       // goto next page
		 oldpage = newpage2;
//		 printf(" tp=%d ",newpage);
		 inteladdress = newpage2*flashpagesize0;  // start address at page bdry
	 	 oldinteladdress = inteladdress;
		 LoadFlashPage(1);   // Load the new page - this may be last record
		                     // So don't overwrite remaining content in page
		 WaitForFlashFree();  // wait for page transfer to complete 250usec
		 flashpageloaded = YES;  // Page loaded so don't do it again next recrd
		 for(i=0; i<newbytecnt; i++)  // Shift record data to start
		      intelbinary[i] = intelbinary[firstbytesonline-newbytecnt+i];
		 bytesonline = newbytecnt;  // Only write new data in record
//		 printf(" newcnt=",newbytecnt);
		 ProgramFlashBuffer(1);  // write remaining record to RAM buffer
		 flashdatainbuffer = YES;
		 totalbytecnt += newbytecnt;
		 flashpageboundarytraversed=NO;
	     }
}

// Return Serial Flash page size
int FlashPageSize(unsigned char sizecode)
{
 	 if((sizecode==AT45DB041ID)||(sizecode==AT45DB081ID))  return 264;
	 else if((sizecode==AT45DB161ID)||(sizecode==AT45DB321ID)) return 528;
	 else if(sizecode==AT45DB642ID) return 1024;
	 else return 0;
}

//  Search for serial flash chips U2, U5
//  U2 is always present while U5 is optional
//  This routine initializes SPI and resets Blackfin
void  CheckFlashTypes()
{
	Release_SPI();
	Init_SPI_Master();
	flashtype0 = SPI_SerialFlashStatus()&FLASHIDMASK;
	flashpagesize0 = FlashPageSize(flashtype0);		
	Release_SPI();
}

	