/**************************************************************

   Program Title:  Led8 Controller
   
   Program Name:  LED8_Uart.c	Version: 0.85
   
   Author:  SS	  			    Date:  August 10, 2005
   
   Compiled with ImageCraft ICCAVR Standard V7.02
   
   This file contains routines that implement basic Usart put/get functions.
   
   Copyright 2004-2006, SOC Robotics, Inc.  All rights reserved.
   
****************************************************************/
/*
 * Copyright 2004-2006, SOC Robotics, Inc.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 * 4. Redistribution is only allowed in combination with SOC Machines, 
 *    Inc. hardware products.
 *
 * THIS SOFTWARE IS PROVIDED BY SOC MACHINES, INC. AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL OC MACHINES, 
 * INC. OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * For additional information see http://www.soc-robotics.com
 */

/*
 * $Log: Wasp_Monitor.c,v $
 * Revision 0.86  2006/02/07 16:40:20  SS
 *    2-wire interface, 8MHz clock, AVR Flash writer, XON/XOFF, 38,700baud
 * Revision 0.52  2005/06/04 18:36:42  SS
 * Preview release
 *
 */
#include <iom48v.h>
#include <macros.h>
#include <string.h>
#include <stdlib.h>
//#include <stdio.h>
#include "LED8_Uart.h"
#include "LED8_Defines.h"
#include "LED8_Extern.h"

void putchar(unsigned char indata)
{
	unsigned char tmphead;
	
	// Calculate buffer index
	tmphead = (UART_TxHead + 1) & UART_TX_BUFFER_MASK; 
		
	// Wait for free space in buffer
	while(tmphead == UART_TxTail) ;
	
	UART_TxBuf[tmphead] = indata; // store data in buffer
	UART_TxHead = tmphead;      // store new index

	// Expand LF to CRLF is textmode globel set
	if((indata==LF)&&(_textmode==1)) {
		// Calculate buffer index
		tmphead = (UART_TxHead + 1) & UART_TX_BUFFER_MASK; 
		
		// Wait for free space in buffer
		while(tmphead == UART_TxTail) ;
	
		UART_TxBuf[tmphead] = CR; // store data in buffer
		UART_TxHead = tmphead;      // store new index
		}

	UCSR0B |= (1<<UDRIE0);        // Fire TX UDRIE interrupt
}

// Read and write functions
unsigned char getcharnw(void)
{
	unsigned char tmptail,rxtmp;

	if(UART_RxHead==UART_RxTail) return 0; // nothing to get
		
	tmptail = (UART_RxTail+1)&UART_RX_BUFFER_MASK;  // calculate buffer index
	UART_RxTail = tmptail;     // store new index
	
	rxcnt--;
	if(rxcnt<0) rxcnt = 0;

	return UART_RxBuf[tmptail];   // return data
}

// Getchar waits until a valid character is received
unsigned char getchar(void)
{
    unsigned char inchar;
	
 	while(!(inchar=getcharnw())) ;
	if(echoinchars==YES) putchar(inchar);
	return inchar;
}

/* Initialize UART */
void InittUART(unsigned char baudrate)
{
 	/* set the baud rate */
	UBRR0H = 0x00;
	UBRR0L = baudrate; 
	UCSR0B = BIT(RXEN0) | BIT(TXEN0); /* enable UART receiver and transmitter */
    //  8 data bits, 2 stop bits
//	UCSRC = (1<<URSEL)|(1<<USBS)|(3<<UCSZ0);
    //  8 data bits, 1 stop bit
	UCSR0C = (1<<UMSEL01)|(3<<UCSZ01);
}

// Polled receive function - non-wait
unsigned char ReceiveByte(void)
{
	if((UCSR0A & (1<<RXB80))) return UDR0;
	else return 0x00;
}

// Polled transmit function
void TransmitByte(unsigned char data)
{
	while ( !(UCSR0A & (1<<UDRE0)) )
		; /* wait for empty transmit buffer */
	UDR0 = data; /* start transmission */
}

//  Buffered write to UART - wait until transmitted
void puttchar(unsigned char data)
{
 	 TransmitByte(data);
}

//  Buffered character read from UART - don't return if 
//  character is Null
unsigned char gettchar(void)
{
     unsigned char inchar;
	 
 	 while(!(inchar=ReceiveByte())) ;
 	 return inchar;
}

void puts(char *outstrg)
{
 	 int i;
	 
	 for(i=0; i<strlen(outstrg); i++)  putchar(outstrg[i]);
}

void putCRLF()
{
 	 puts("\n\r");
}

//  Load string constant from Flash (conserves RAM)
void putfs(const char *signon1)
{ 	
	char inchar;
    int i;
	
	i = 0;
	while((inchar=*(signon1+i++))!=0x00)  putchar(inchar);
}

